/*DDK*************************************************************************/
/*                                                                           */
/* COPYRIGHT    Copyright (C) 1995 IBM Corporation                           */
/*                                                                           */
/*    The following IBM OS/2 WARP source code is provided to you solely for  */
/*    the purpose of assisting you in your development of OS/2 WARP device   */
/*    drivers. You may use this code in accordance with the IBM License      */
/*    Agreement provided in the IBM Device Driver Source Kit for OS/2. This  */
/*    Copyright statement may not be removed.                                */
/*                                                                           */
/*****************************************************************************/
#pragma title("CLIPRC.C")

#define INCL_DEV             // include all DEV definitions
#define INCL_WIN             // include Win definitions
#define INCL_GPI             // include all Gpi definitions
#define INCL_GRE             // include all Gre definitions
#define INCL_GRE_CLIP
#define INCL_ERRORS          // include all Gre error definitions

#include <os2.h>
#include <string.h>
#include <stdlib.h>
#include <pmddi.h>              // needed for GRE function definitions.
#include <stdio.h>
#include "DTTDLLDF.h"           // test case defines.
#include "common1.h"            // Graphics testing common file.
#include "common2.h"
#define EXTERN extern           // for globals in .h to be extern defs
#include "greclip.h"            // Test Definition File.

/****************************************************************************
 *
 * Subtask Name: GreClip.dll
 *
 * Module name:  clipexh.c
 *
 * Purpose:      module for GRECLIP exhautive entry points
 *
 * Revision Log: 12/17/91,      Graphics Suite Extension, Thom Bain
 *                               Initial Design.
 *
 * Documentation Reference:  Section 13.4
 *
 * Contents:     r2f - convert a real/double to fixed point notation
 *               setup_overflow - setup so GpiQueryClipBox gets
 *                                                PMERR_COORDINATE_OVERFLOW
 *               reset_overflow - restore presentaion page, model transform
 *                                  and page viewport after setup_overflow
 *               GreGetClipBoxRc
 *               GreGetClipRectsRc
 *               GreOffsetClipRegionRc
 *               GrePtVisibleRc
 *               GreRectVisibleRc
 *               GreQueryClipRegionRc
 *               GreSelectClipRegionRc
 *               GreIntersectClipRectangleRc
 *               GreExcludeClipRectangleRc
 *               GreSetXformRectRc
 *               GreSaveRegionRc
 *               GreRestoreRegionRc
 *               GreSelectPathRegionRc
 *               GreRegionSelectBitmapRc
 *               GreClipPathCurvesRc - stubbed
 *               GreCopyClipRegionRc
 *
****************************************************************************/

/****************************************************************************
 *
 * Routine Name: r2f
 *
 * Purpose: support function for converting double (floating point) value
 *          to a fixed point representation for transform matrix values.
 *
 * System Requirements: Section
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    double     d          // value to convert.
 *
 * Outputs:
 *          fixed point representation of "d"
 *
 * Subroutines Required: NONE
 *
 * Limitations:  NONE
 *
 ***************************************************************************/

/***************************************************************************
 * NOTE: that fixed point is a thirty two bit structure representing a
 *       floating point number where the most significant 16 bits is the
 *       whole part of the number and the least significant 16 bits is the
 *       fraction part where the fraction is the (unsigned) value over
 *       65536. Also the final value is the sum of the two fields; i.e.
 *       -1.2 is stored as -2.8)
 *
 * Algorithm:
 * {
 *      get the whole part of the double
 *      if the whole part is negative
 *        value = whole part minus one plus one minus the fraction * 65536
 *      else
 *        value = whole part plut the fraction part times 65536
 *      return the value.
 * }
 *
 ***************************************************************************/

FIXED r2f( double d)
{
   FIXED value;
   LONG  whole_part;

   whole_part = (LONG)d;
   // if negative subtract one from whole part and make fraction
   // (1 - fraction)
   if (whole_part < 0)
   {
     value = ((whole_part - 1) << 16) |
              (LONG)((1.0 - (d - (double)whole_part)) * 65536);
   }
   else
   {
     value = (whole_part << 16) | (LONG)(d - (double)whole_part) * 65536;
   }
   return(value);
}

/****************************************************************************
 *
 * Routine Name: setup_overflow
 *
 * Purpose: support function for testing PMERR_COORDINATE_OVERFLOW for the
 *          GreGetClipBoxRc entry point
 *
 * System Requirements: Section
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    PCHAR          // entry point name for error reporting
 *
 * Outputs: BOOL success flag if the setup works or not.
 *          new page (size), model transform, and page viewport that will
 *          cause device coordinates to exceed maximum legal world/page
 *          coordinate value (134217727)
 *
 * Subroutines Required: GpiQueryPS, SHOWERROR, GpiConvert, GpiSetPS, r2f,
 *         ERRSTATUS, LOGINFO, GpiSetModelTransformMatrix, GpiSetPageviewport
 *
 * Limitations:
 *          Setup for PMERR_COORDINATE_OVERFLOW requires changing the page
 *          size via GpiSetPS which fails with PMERR_INV_DC_TYPE for
 *          OD_QUEUED/PM_Q_STD, OD_MEMORY, OD_METAFILE & OD_METAFILENOQUERY
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query the PS to get the options value for changing page (this
 *        maintains the DTT configuration selection values)
 *      - determine the page size that maps to the maximum positive legal
 *        device coordinates.
 *      - attempt to set new page
 *      - if failed return FALSE so test can be skipped (log this fact now)
 *      - set a model transform that maps the new page to the maximum legal
 *        world coordinates plus one
 *      - set the page viewport to the maximum positive device coordinates
 *        (origin = [0, 0] and upper right corner = [32767, 32767])
 *      - return TRUE to indicate success.
 * }
 *
 ***************************************************************************/

BOOL setup_overflow(HPS hPS, HWND hWndClient, PCHAR EntryName)
{
  BOOL      bSuccess;
  MATRIXLF  matlfModel;
  double    Xsx, Xsy;
  RECTL     rcRect;
  ULONG     flOptions;
  SIZEL     sizlSize;
  POINTL    ptl;

  // get current PS size and options.
  flOptions = GpiQueryPS(hPS, &sizlSize);

  // find max page size that won't overflow device coord. limits
  ptl.x = ptl.y = 32767L;
  if (!GpiConvert(hPS, CVTC_DEVICE, CVTC_PAGE, 1L, &ptl))
    SHOWERROR("GpiConvert", NULL);

  // set new page to this size.
  sizlSize.cx = ptl.x;
  sizlSize.cy = ptl.y;
  if(!GpiSetPS(hPS, &sizlSize, flOptions))
  {
    ErrorCount++; // force error to show
    ERRSTATUS(TRUE, "GpiSetPS", NULL);
    ErrorCount--; // but don't let test fail
    ERRSTATUS(FALSE, "", NULL);  // reset FAIL for test
    LOGINFO(L_FAIL,"@","Test skipped; environment restriction on setup?");
    return(FALSE);
  }

  // setup a model transform that causes page space to map to maximum
  // legal world coordinates plus one (134217728L)
  Xsx = (double)ptl.x / 134217728.0;
  Xsy = (double)ptl.y / 134217728.0;

  matlfModel.fxM11 = r2f(Xsx);              // x scale factor
  matlfModel.fxM12 = (FIXED)0x00000000;     // x shear factor
  matlfModel.lM13  = 0L;
  matlfModel.fxM21 = (FIXED)0x00000000;     // y shear factor
  matlfModel.fxM22 = r2f(Xsy);              // y scale factor
  matlfModel.lM23  = 0L;
  matlfModel.lM31  = 0L;                    // x translation factor
  matlfModel.lM32  = 0L;                    // y translation factor
  matlfModel.lM33  = 1L;
  bSuccess =
     GpiSetModelTransformMatrix( hPS, 9L, &matlfModel, TRANSFORM_REPLACE );
  if (bSuccess == FALSE) {
    SHOWERROR("GpiSetModelTransformMatrix", EntryName);
  }

  // set page viewport so the device transform changes such that
  // above transform will cause overflow
  rcRect.xLeft = 0L;
  rcRect.xRight = 32767L;
  rcRect.yBottom = 0L;
  rcRect.yTop = 32767L;
  if (GpiSetPageViewport(hPS, &rcRect) == FALSE)
   SHOWERROR("GpiSetPageViewport", NULL);

  return(TRUE);
} // end setup_overflow

/****************************************************************************
 *
 * Routine Name: reset_overflow
 *
 * Purpose: support function to restore the PS, the model transform, and
 *  page viewport to original (default) DTT created values.
 *
 * System Requirements: Section
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    PCHAR          // entry point name for error reporting
 *
 * Outputs: PS, model transform to unity, and page viewport to device size.
 *
 * Subroutines Required: GpiSetPageViewport, GpiQueryPS, GpiSetPS, and
 *                       GpiSetModelTransformMatrix.
 *
 * Limitations: NONE
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - restore page viewport to actual device size.
 *      - query the PS for the options field to use in resetting page
 *      - set the model transform matrix back to unity
 *      - set the page back to its original size as created by DTT.
 * }
 *
 ***************************************************************************/

VOID reset_overflow(HPS hPS, HWND hWndClient, PCHAR EntryName)
{
  MATRIXLF  matlfModel;
  BOOL      bSuccess;
  RECTL     rcRect;
  ULONG     flOptions;
  SIZEL     sizlSize;

  // reset to default viewport
  rcRect.xLeft = 0L;
  rcRect.xRight = dev_width - 1;
  rcRect.yBottom = 0L;
  rcRect.yTop = dev_height - 1;

  if (GpiSetPageViewport(hPS, &rcRect) == FALSE)
   SHOWERROR("GpiSetPageViewport", NULL);

  // get current PS size and options.
  flOptions = GpiQueryPS(hPS, &sizlSize);

  // reset to unity transform
  bSuccess =
     GpiSetModelTransformMatrix( hPS, 0L, &matlfModel, TRANSFORM_REPLACE );
  if(bSuccess == FALSE) {
    SHOWERROR("GpiSetModelTransformMatrix", EntryName);
  }

  // reset new page to original size.
  sizlSize.cx = pg_width;
  sizlSize.cy = pg_height;
  if(!GpiSetPS(hPS, &sizlSize, flOptions))
    SHOWERROR("GpiSetPS", NULL);

}

/****************************************************************************
 *
 * Routine Name: GreGetClipBoxRc
 *
 * Purpose: return code test for GreGetClipBoxRc
 *
 * System Requirements: Section 13.5.1
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, setup_overflow, reset_overflow, GreGetClipBox,
 *    MAKEPSBUSY, FREEPS, and DeletePrinterOutput.
 *
 * Limitations:
 *          Setup for PMERR_COORDINATE_OVERFLOW requires changing the page
 *          size via GpiSetPS which fails with PMERR_INV_DC_TYPE for
 *          OD_QUEUED/PM_Q_STD, OD_MEMORY, OD_METAFILE & OD_METAFILENOQUERY
 *
 *          HDC_BUSY fails for OD_INFO, OD_MEMORY, OD_METAFILE, and
 *          OD_METAFILENOQUERY. OD_MEMORY causes DTT to trap or forces the
 *          user to terminate and restart to continue.
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - if setup_overflow succeeds
 *        - call the function under test and pass result to CHECKRC to log
 *          return code check for PMERR_COORDINATE_OVERFLOW
 *        - call reset_overflow to restore environment
 *      - Call MAKEPSBUSY then the function under test, then FREEPS and
 *        pass function call result to CHECKRC to log return code check
 *        for PMERR_HDC_BUSY.
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreGetClipBoxRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreGetClipBoxRc"; // entry point name
  HDC   hDC;
  HAB   hAB;
  BOOL  bSuccess;
  RECTL rcRect;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_HDC_BUSY
        PMERR_DEV_FUNC_NOT_INSTALLED - not tested
        PMERR_COORDINATE_OVERFLOW - see limitations
        PMERR_INV_LENGTH_OR_COUNT - not tested
        PMERR_INV_RECT - not tested
        PMERR_INV_HDC
        PMERR_INV_COORD_SPACE - not tested
      *******************************************************************/
    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = (GreGetClipBox( (HDC)NULL, &rcRect) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = (GreGetClipBox( (HDC)-1L, &rcRect) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_COORDINATE_OVERFLOW, EntryName);
    if (setup_overflow(hPS, hWndClient, EntryName))
    {
      // set a transform that will cause dev_width or dev_height to
      // overflow as world coordinates
      bSuccess = (GreGetClipBox( hDC, &rcRect) != RGN_ERROR);
      CHECKRC(bSuccess, PMERR_COORDINATE_OVERFLOW, "");
      reset_overflow(hPS, hWndClient, EntryName);
    }
    SUBEXIT();

    SUBENTER(PMERR_HDC_BUSY, EntryName);
    MAKEPSBUSY(hPS);               //Make this PS busy.
    bSuccess = (GreGetClipBox( hDC, &rcRect) != RGN_ERROR);
    FREEPS;                       //Kill BUSY thread.
    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");
    SUBEXIT();

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreGetClipRectsRc
 *
 * Purpose: return code test for GreGetClipRects function
 *
 * System Requirements: Section 13.5.2
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GreGetClipRects, MAKEPSBUSY, FREEPS, and
 *    DeletePrinterOutput.
 *
 * Limitations:
 *          NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - initialize control structure to return one rectangle at a time
 *        from top left to bottom right
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - set bounds rectangle to have xRight < xLeft and yTop < yBottom
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_RECT
 *      - set the direction flag to 0L
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_REGION_CONTROL
 *      - Call MAKEPSBUSY then the function under test, then FREEPS and
 *        pass function call result to CHECKRC to log return code check
 *        for PMERR_HDC_BUSY.
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreGetClipRectsRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreGetClipRectsRc"; // entry point name
  HDC   hDC;
  HAB   hAB;
  BOOL  bSuccess;
  RECTL rcBound, rcRects[1];
  RGNRECT Control;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
          if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    Control.ircStart = 1;
    Control.crc = 1;
    Control.ulDirection = RECTDIR_LFRT_TOPBOT;

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_HDC_BUSY
        PMERR_DEV_FUNC_NOT_INSTALLED - not tested
        PMERR_INV_RECT
        PMERR_INV_HDC
        PMERR_INV_REGION_CONTROL
      *******************************************************************/
    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = GreGetClipRects( (HDC)NULL, NULL, &Control, (PRECTL)&rcRects);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = GreGetClipRects( (HDC)-1L, NULL, &Control, (PRECTL)&rcRects);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_RECT, EntryName);
    rcBound.xLeft = dev_width;
    rcBound.xRight = 0L;
    rcBound.yBottom = dev_height;
    rcBound.yTop = 0L;
    bSuccess = GreGetClipRects( hDC, &rcBound, &Control, (PRECTL)&rcRects);
    CHECKRC(bSuccess, PMERR_INV_RECT, "On Bounds Rectangle");
    SUBEXIT();

    SUBENTER(PMERR_INV_COORDINATE, EntryName);
    rcBound.xLeft = 0L;
    rcBound.xRight = 32768L;
    rcBound.yBottom = 0L;
    rcBound.yTop = 32768L;
    bSuccess = GreGetClipRects( hDC, &rcBound, &Control, (PRECTL)&rcRects);
    CHECKRC(bSuccess, PMERR_INV_COORDINATE, "Bound > Device Coord Limits");
    SUBEXIT();

    SUBENTER(PMERR_INV_REGION_CONTROL, EntryName);
    Control.ulDirection = 0L;
    bSuccess = GreGetClipRects( hDC, NULL, &Control, (PRECTL)&rcRects);
    CHECKRC(bSuccess, PMERR_INV_REGION_CONTROL, "Invalid Direction");
    SUBEXIT();

    SUBENTER(PMERR_HDC_BUSY, EntryName);
    MAKEPSBUSY(hPS);               //Make this PS busy.
    bSuccess = GreGetClipRects( hDC, NULL, &Control, (PRECTL)&rcRects);
    FREEPS;                       //Kill BUSY thread.
    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");
    SUBEXIT();

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreOffsetClipRegionRc
 *
 * Purpose: return code test for GreOffsetClipRegion functin
 *
 * System Requirements: Section 13.5.3
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GpiIntersectClipRectangle, GpiConvert, GpiBeginArea,
 *    GpiEndArea, GpiBeginPath, GpiEndPath, GreOffsetClipRegion, MAKEPSBUSY,
 *    FREEPS, GpiSetClipRegion, GpiDestroyRegion, and DeletePrinterOutput.
 *
 * Limitations:
 *          NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - create a valid region to offset, make it page size to give busy
 *        check chance to fail.
 *      - initialize offset amount to a minimal value and determine the
 *        maximum world coordinate that maps to the maximum valid device
 *        coordinates (32767).
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - start an area (GpiBeginArea)
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_IN_AREA
 *      - end the area (GpiEndArea)
 *      - start a path (GpiBeginPath)
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_IN_PATH
 *      - end the path (GpiEndPath)
 *      - set offset such that move amount in y direction will cause top
 *        of current region to move one unit past max world calculated
 *        above.
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_COORDINATE_OVERFLOW
 *      - move the region down so there is enough room to attempt move
 *        by more the the max world value
 *      - attempt to move by more than max world value and pass result to
 *        CHECKRC to log return code check for PMERR_INV_COORD_OFFSET
 *      - Call MAKEPSBUSY then the function under test, then FREEPS and
 *        pass function call result to CHECKRC to log return code check
 *        for PMERR_HDC_BUSY.
 *      - cleanup: deselect active region and destroy it.
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreOffsetClipRegionRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static   PCHAR EntryName="GreOffsetClipRegionRc"; // entry point name
  HDC      hDC;
  HAB      hAB;
  BOOL     bSuccess;
  HRGN     hrgn;
  POINTL   ptlOffset, ptlMaxWorld;
  RECTL    rcRect;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
    if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    // create a region to offset
    rcRect.xLeft = 0L;
    rcRect.xRight = pg_width - 1;
    rcRect.yBottom = 0L;
    rcRect.yTop = pg_height;
    if (GpiIntersectClipRectangle(hPS, &rcRect) == RGN_ERROR)
    {
      CLEANUP("GpiIntersectClipRectangle", NULL);
      return;
    }

    ptlOffset.x = 0L;
    ptlOffset.y = 10L;
    ptlMaxWorld.x = ptlMaxWorld.y = 32767;
    GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 1L, &ptlMaxWorld);

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_INV_IN_AREA
        PMERR_INV_IN_PATH
        PMERR_HDC_BUSY
        PMERR_DEV_FUNC_NOT_INSTALLED - not tested
        PMERR_COORDINATE_OVERFLOW
        PMERR_INSUFFICIENT_MEMORY
        PMERR_INV_LENGTH_OR_COUNT
        PMERR_INV_RECT
        PMERR_INV_COORDINATE
        PMERR_INV_HDC
        PMERR_INV_REGION_CONTROL
        PMERR_INV_COORD_OFFSET
        PMERR_INV_COORD_SPACE
      *******************************************************************/
    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = (GreOffsetClipRegion( (HDC)NULL, &ptlOffset) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = (GreOffsetClipRegion( (HDC)-1L, &ptlOffset) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_IN_AREA, EntryName);
    if (GpiBeginArea(hPS, BA_WINDING) == FALSE)
    {
      /* setup errored */
      ErrorCount++;
      ERRSTATUS(TRUE,"GpiBeginArea", NULL);
    }
    else
    {
      bSuccess = (GreOffsetClipRegion(hDC, &ptlOffset) != RGN_ERROR);
      CHECKRC(bSuccess, PMERR_INV_IN_AREA, NULL);
      if (GpiEndArea(hPS) == FALSE)
      {
        /* close up failed */
        ErrorCount++;
        ERRSTATUS(TRUE,"GpiEndArea", NULL);
      }
    }
    SUBEXIT();

    SUBENTER(PMERR_INV_IN_PATH, EntryName);
    if (GpiBeginPath(hPS, 1L) == FALSE)
    {
      /* setup errored */
      ErrorCount++;
      ERRSTATUS(TRUE,"GpiBeginPath", NULL);
    }
    else
    {
      bSuccess = (GreOffsetClipRegion(hDC, &ptlOffset) != RGN_ERROR);
      CHECKRC(bSuccess, PMERR_INV_IN_PATH, NULL);
      if (GpiEndPath(hPS) == FALSE)
      {
        /* close up failed */
        ErrorCount++;
        ERRSTATUS(TRUE,"GpiEndPath", NULL);
      }
    }
    SUBEXIT();

    SUBENTER(PMERR_COORDINATE_OVERFLOW, EntryName);
    // attempt to move region out of valid device coord space
    ptlOffset.y = ptlMaxWorld.y - rcRect.yTop + 1L;
    bSuccess = (GreOffsetClipRegion( hDC, &ptlOffset) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_COORDINATE_OVERFLOW, NULL);
    SUBEXIT();

    SUBENTER(PMERR_INV_COORD_OFFSET, EntryName);
    // move region down enough so can be moved without OVERFLOW
    ptlOffset.x = 0L;
    ptlOffset.y = - pg_height;
    GreOffsetClipRegion( hDC, &ptlOffset);
    ptlOffset.y = ptlMaxWorld.y + pg_height;
    bSuccess = (GreOffsetClipRegion( hDC, &ptlOffset) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_COORD_OFFSET, NULL);
    SUBEXIT();

    SUBENTER(PMERR_HDC_BUSY, EntryName);
    ptlOffset.y = 10L;
    MAKEPSBUSY(hPS);               //Make this PS busy.
    bSuccess = (GreOffsetClipRegion( hDC, &ptlOffset) != RGN_ERROR);
    FREEPS;                       //Kill BUSY thread.
    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");
    SUBEXIT();

    // cleanup
    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
      SHOWERROR("GpiSetClipRegion", NULL);
    if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GrePtVisibleRc
 *
 * Purpose: return code test for GrePtVisible function
 *
 * System Requirements: Section 13.5.4
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GpiConvert, LOGINFO, GrePtVisible, MAKEPSBUSY,
 *    FREEPS, and DeletePrinterOutput.
 *
 * Limitations:
 *          HDC_BUSY fails for OD_INFO, OD_METAFILE, OD_METAFILENOQUERY,
 *          and OD_MEMORY. OD_MEMEORY causes DTT to trap
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - create a valid region to offset, make it page size to give busy
 *        check chance to fail.
 *      - initialize point to a valid value: (0, 0)
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Call MAKEPSBUSY then the function under test, then FREEPS and
 *        pass function call result to CHECKRC to log return code check
 *        for PMERR_HDC_BUSY.
 *      - set point coordinate to one more than max legal value (134217727)
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_COORDINATE_OVERFLOW
 *      - set point to be world space equivalent to one more than max valid
 *        device coordinate (use GpiConvert from device to world of 32768)
 *      - if GpiConvert succeeded
 *        - call the function under test with this point and pass result to
 *          CHECKRC to log return code check for PMERR_INV_COORDINATE
 *        else
 *        - log that PMERR_INV_COORDINATE skipped as could not create the
 *          error for this device size and resolution.
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GrePtVisibleRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GrePtVisibleRc"; // entry point name
  HDC   hDC;
  HAB   hAB;
  BOOL  bSuccess;
  POINTL ptlPoint;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
          if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    // initialize point to something valid for HDC_BUSY and INV_HDC
    ptlPoint.x = 0L;
    ptlPoint.y = 0L;

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_HDC_BUSY
        PMERR_DEV_FUNC_NOT_INSTALLED - not tested
        PMERR_COORDINATE_OVERFLOW
        PMERR_INV_LENGTH_OR_COUNT - not tested
        PMERR_INV_RECT - not tested
        PMERR_INV_COORDINATE
        PMERR_INV_HDC
        PMERR_INV_COORD_SPACE - not tested
      *******************************************************************/
    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = (GrePtVisible( (HDC)NULL, &ptlPoint) != PVIS_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = (GrePtVisible( (HDC)-1L, &ptlPoint) != PVIS_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_HDC_BUSY, EntryName);
    MAKEPSBUSY(hPS);               //Make this PS busy.
    bSuccess = (GrePtVisible( hDC, &ptlPoint) != PVIS_ERROR);
    FREEPS;                       //Kill BUSY thread.
    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");
    SUBEXIT();

    SUBENTER(PMERR_COORDINATE_OVERFLOW, EntryName);
    ptlPoint.x = 134217728L;
    bSuccess = (GrePtVisible( hDC, &ptlPoint) != PVIS_ERROR);
    CHECKRC(bSuccess, PMERR_COORDINATE_OVERFLOW, "");
    SUBEXIT();

    SUBENTER(PMERR_INV_COORDINATE, EntryName);
    // determine maximum valid world coordinate plus one device unit
    ptlPoint.x = 32768L;
    ptlPoint.y = 32768L;
    if (GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 1L, &ptlPoint))
    {
      bSuccess = (GrePtVisible( hDC, &ptlPoint) != PVIS_ERROR);
      CHECKRC(bSuccess, PMERR_INV_COORDINATE, "");
    }
    else
    {
      // clear the DTT error in the queue
      WinGetLastError(hAB);
      LOGINFO(L_TRACE, "*",
               "Test skipped! Cannot create error w/current configuration");
    }
    SUBEXIT();

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreRectVisibleRc
 *
 * Purpose: return code test for GreRectVisible function
 *
 * System Requirements: Section 13.5.5
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GpiConvert, LOGINFO, GreRectVisible, MAKEPSBUSY,
 *    FREEPS, and DeletePrinterOutput.
 *
 * Limitations:
 *          NONE but note that PMERR_COORDINATE_OVERFLOW attempts always
 *          return PMERR_INV_COORDINATE so testing for it was omitted.
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - create a valid region to offset, make it page size to give busy
 *        check chance to fail.
 *      - initialize rectangle to a valid value: (page size)
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Call MAKEPSBUSY then the function under test, then FREEPS and
 *        pass function call result to CHECKRC to log return code check
 *        for PMERR_HDC_BUSY.
 *      - set rectangle to have xRight < xLeft
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_RECT
 *      - set rectangle upper right corner to be world space equivalent to
 *        one more than max valid device coordinate (use GpiConvert from
 *        device to world of 32768)
 *      - if GpiConvert succeeded
 *        - call the function under test with this point and pass result to
 *          CHECKRC to log return code check for PMERR_INV_COORDINATE
 *        else
 *        - log that PMERR_INV_COORDINATE skipped as could not create the
 *          error for this device size and resolution.
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreRectVisibleRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreRectVisibleRc"; // entry point name
  HDC   hDC;
  HAB   hAB;
  BOOL  bSuccess;
  RECTL rcRect;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
        if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    // initialize to something valid for INV_HDC and HDC_BUSY tests
    rcRect.xLeft = 0L;
    rcRect.xRight = pg_width - 1;
    rcRect.yBottom = 0L;
    rcRect.yTop = pg_height - 1;

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_HDC_BUSY
        PMERR_DEV_FUNC_NOT_INSTALLED - not tested
        PMERR_COORDINATE_OVERFLOW - not tested
        PMERR_INV_LENGTH_OR_COUNT - not tested
        PMERR_INV_RECT
        PMERR_INV_COORDINATE
        PMERR_INV_HDC
        PMERR_INV_COORD_SPACE - not tested
      *******************************************************************/
    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = (GreRectVisible( (HDC)NULL, &rcRect) != RVIS_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = (GreRectVisible( (HDC)-1L, &rcRect) != RVIS_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_HDC_BUSY, EntryName);
    MAKEPSBUSY(hPS);               //Make this PS busy.
    bSuccess = (GreRectVisible( hDC, &rcRect) != RVIS_ERROR);
    FREEPS;                       //Kill BUSY thread.
    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");
    SUBEXIT();

    SUBENTER(PMERR_INV_RECT, EntryName);
    rcRect.xRight = - pg_width;
    bSuccess = (GreRectVisible( hDC, &rcRect) != RVIS_ERROR);
    CHECKRC(bSuccess, PMERR_INV_RECT, "");
    SUBEXIT();

    SUBENTER(PMERR_INV_COORDINATE, EntryName);
    // determine maximum valid world coordinate plus one device unit
    rcRect.xRight = 32768L;
    rcRect.yTop = 32768L;
    if(GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L, (PPOINTL)&rcRect))
    {
      bSuccess = (GreRectVisible( hDC, &rcRect) != RVIS_ERROR);
      CHECKRC(bSuccess, PMERR_INV_COORDINATE, "");
    }
    else
    {
      // clear the DTT error in the queue
      WinGetLastError(hAB);
      LOGINFO(L_TRACE, "*",
               "Test skipped! Cannot create error w/current configuration");
    }
    SUBEXIT();

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreQueryClipRegionRc
 *
 * Purpose: return code test for GreQueryClipRegion function
 *
 * System Requirements: Section 13.5.6
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GreQueryClipRegion, MAKEPSBUSY, FREEPS, and
 *    DeletePrinterOutput.
 *
 * Limitations:
 *          NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Call MAKEPSBUSY then the function under test, then FREEPS and
 *        pass function call result to CHECKRC to log return code check
 *        for PMERR_HDC_BUSY.
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreQueryClipRegionRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreQueryClipRegionRc"; // entry point name
  HDC   hDC;
  HAB   hAB;
  BOOL  bSuccess;
  HRGN  hrgn;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
          if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_HDC_BUSY
        PMERR_INV_HDC
      *******************************************************************/
    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    hrgn = GreQueryClipRegion( (HDC)NULL );
    bSuccess = (hrgn != HRGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    hrgn = GreQueryClipRegion( (HDC)-1L );
    bSuccess = (hrgn != HRGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_HDC_BUSY, EntryName);
    MAKEPSBUSY(hPS);               //Make this PS busy.
    hrgn = GreQueryClipRegion(hDC);
    bSuccess = (hrgn != HRGN_ERROR);
    FREEPS;                       //Kill BUSY thread.
    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");
    SUBEXIT();

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreSelectClipRegionRc
 *
 * Purpose: return code test for GreSelectClipRegion function
 *
 * System Requirements: Section 13.5.7
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GpiIntersectClipRectangle, GpiSetClipRegion,
 *    GpiDestroyRegion, GreSelectClipRegion, GpiBeginArea, GpiEndArea,
 *    GpiBeginPath, GpiEndPath, MAKEPSBUSY, FREEPS, and DeletePrinterOutput.
 *
 * Limitations:
 *          NONE but could not generate PMERR_REGION_IS_CLIP_REGION ???????
 *            and testing for it is omitted.
 *          also: no amount of setup using new page sizes, viewport, region
 *          sizes and transforms could cause PMERR_INV_COORDINATE
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - create a valid region
 *      - Call MAKEPSBUSY then the function under test, then FREEPS and
 *        pass function call result to CHECKRC to log return code check
 *        for PMERR_HDC_BUSY.
 *      - cleanup: deselect the region and destroy it.
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - start an area (GpiBeginArea)
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_IN_AREA
 *      - end the area (GpiEndArea)
 *      - start a path (GpiBeginPath)
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_IN_PATH
 *      - end the path (GpiEndPath)
 *      - set a region handle to -1L
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_HRGN
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreSelectClipRegionRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreSelectClipRegionRc"; // entry point name
  HDC   hDC;
  HAB   hAB;
  BOOL  bSuccess;
  HRGN  hrgn, hrgn2;
  RECTL rcRect;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
        if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_INV_IN_AREA
        PMERR_INV_IN_PATH
        PMERR_HDC_BUSY
        PMERR_DEV_FUNC_NOT_INSTALLED - not tested
        PMERR_INSUFFICIENT_MEMORY
        PMERR_INV_RECT
        PMERR_INV_COORDINATE
        PMERR_INV_HDC
        PMERR_INV_HRGN
        PMERR_HRGN_BUSY
        PMERR_REGION_IS_CLIP_REGION
        PMERR_INV_REGION_CONTROL
      *******************************************************************/

    SUBENTER(PMERR_HDC_BUSY, EntryName);

    // create a clip region
    rcRect.xLeft = 0L;
    rcRect.xRight = pg_width - 1;
    rcRect.yBottom = 0L;
    rcRect.yTop = pg_height - 1;
    if (GpiIntersectClipRectangle(hPS, &rcRect) == RGN_ERROR)
      SHOWERROR("GpiIntersectClipRectangle", NULL);

    MAKEPSBUSY(hPS);               //Make this PS busy.
    bSuccess = (GreSelectClipRegion( hDC, NULL, &hrgn) != RGN_ERROR);
    FREEPS;                       //Kill BUSY thread.
    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");

    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
      SHOWERROR("GpiSetClipRegion", NULL);
    if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);

    SUBEXIT();

    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = (GreSelectClipRegion( (HDC)NULL, NULL, &hrgn) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = (GreSelectClipRegion( (HDC)-1L, NULL, &hrgn) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_IN_AREA, EntryName);
    if (GpiBeginArea(hPS, BA_WINDING) == FALSE)
    {
      /* setup errored */
      ErrorCount++;
      ERRSTATUS(TRUE,"GpiBeginArea", NULL);
    }
    else
    {
      bSuccess = (GreSelectClipRegion(hDC, NULL, &hrgn) != RGN_ERROR);
      CHECKRC(bSuccess, PMERR_INV_IN_AREA, NULL);
      if (GpiEndArea(hPS) == FALSE)
      {
        /* close up failed */
        ErrorCount++;
        ERRSTATUS(TRUE,"GpiEndArea", NULL);
      }
    }
    SUBEXIT();

    SUBENTER(PMERR_INV_IN_PATH, EntryName);
    if (GpiBeginPath(hPS, 1L) == FALSE)
    {
      /* setup errored */
      ErrorCount++;
      ERRSTATUS(TRUE,"GpiBeginPath", NULL);
    }
    else
    {
      bSuccess = (GreSelectClipRegion(hDC, NULL, &hrgn) != RGN_ERROR);
      CHECKRC(bSuccess, PMERR_INV_IN_PATH, NULL);
      if (GpiEndPath(hPS) == FALSE)
      {
        /* close up failed */
        ErrorCount++;
        ERRSTATUS(TRUE,"GpiEndPath", NULL);
      }
    }
    SUBEXIT();

    SUBENTER(PMERR_INV_HRGN, EntryName);
    hrgn = (HRGN)-1L;
    bSuccess = (GreSelectClipRegion( hDC, hrgn, &hrgn2) != RGN_ERROR);
    CHECKRC(bSuccess,PMERR_INV_HRGN,"");
    SUBEXIT();

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreIntersectClipRectangleRc
 *
 * Purpose: return code test for GreIntersectClipRectangle function
 *
 * System Requirements: Section 13.5.8
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GreIntersectClipRectangle, GpiSetClipRegion,
 *    GpiDestroyRegion, GpiBeginArea, GpiEndArea, GpiBeginPath, GpiEndPath,
 *    MAKEPSBUSY, FREEPS, and DeletePrinterOutput.
 *
 * Limitations:
 *          could not generate PMERR_COORDINATE_OVERFLOW, test omitted.
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - initialize a rectangle to the page size
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - start an area (GpiBeginArea)
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_IN_AREA
 *      - end the area (GpiEndArea)
 *      - start a path (GpiBeginPath)
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_IN_PATH
 *      - end the path (GpiEndPath)
 *      - Call MAKEPSBUSY then the function under test, then FREEPS and
 *        pass function call result to CHECKRC to log return code check
 *        for PMERR_HDC_BUSY.
 *      - set rectangle to have xRight < xLeft
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_RECT
 *      - set rectangle upper right corner to be world space equivalent to
 *        one more than max valid device coordinate (use GpiConvert from
 *        device to world of 32768)
 *      - if GpiConvert succeeded
 *        - call the function under test with this point and pass result to
 *          CHECKRC to log return code check for PMERR_INV_COORDINATE
 *        else
 *        - log that PMERR_INV_COORDINATE skipped as could not create the
 *          error for this device size and resolution.
 *      - cleanup: deselect any active region and destroy it.
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreIntersectClipRectangleRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreIntersectClipRectangleRc"; // entry point name
  HDC   hDC;
  HAB   hAB;
  BOOL  bSuccess;
  HRGN  hrgn;
  RECTL rcRect;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
          if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    // initialize to something valid for INV_HDC and HDC_BUSY tests
    rcRect.xLeft = 0L;
    rcRect.xRight = pg_width - 1;
    rcRect.yBottom = 0L;
    rcRect.yTop = pg_height - 1;

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_INV_IN_AREA
        PMERR_INV_IN_PATH
        PMERR_HDC_BUSY
        PMERR_DEV_FUNC_NOT_INSTALLED - not tested
        PMERR_COORDINATE_OVERFLOW
        PMERR_INSUFFICIENT_MEMORY
        PMERR_INV_LENGTH_OR_COUNT
        PMERR_INV_RECT
        PMERR_INV_COORDINATE
        PMERR_INV_HDC
        PMERR_INV_HRGN
        PMERR_INV_REGION_CONTROL
        PMERR_INV_COORD_SPACE
      *******************************************************************/
    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = (GreIntersectClipRectangle( (HDC)NULL, &rcRect) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = (GreIntersectClipRectangle( (HDC)-1L, &rcRect) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_IN_AREA, EntryName);
    if (GpiBeginArea(hPS, BA_WINDING) == FALSE)
    {
      /* setup errored */
      ErrorCount++;
      ERRSTATUS(TRUE,"GpiBeginArea", NULL);
    }
    else
    {
      bSuccess = (GreIntersectClipRectangle(hDC, &rcRect) != RGN_ERROR);
      CHECKRC(bSuccess, PMERR_INV_IN_AREA, NULL);
      if (GpiEndArea(hPS) == FALSE)
      {
        /* close up failed */
        ErrorCount++;
        ERRSTATUS(TRUE,"GpiEndArea", NULL);
      }
    }
    SUBEXIT();

    SUBENTER(PMERR_INV_IN_PATH, EntryName);
    if (GpiBeginPath(hPS, 1L) == FALSE)
    {
      /* setup errored */
      ErrorCount++;
      ERRSTATUS(TRUE,"GpiBeginPath", NULL);
    }
    else
    {
      bSuccess = (GreIntersectClipRectangle(hDC, &rcRect) != RGN_ERROR);
      CHECKRC(bSuccess, PMERR_INV_IN_PATH, NULL);
      if (GpiEndPath(hPS) == FALSE)
      {
        /* close up failed */
        ErrorCount++;
        ERRSTATUS(TRUE,"GpiEndPath", NULL);
      }
    }
    SUBEXIT();

    SUBENTER(PMERR_HDC_BUSY, EntryName);
    MAKEPSBUSY(hPS);               //Make this PS busy.
    bSuccess = (GreIntersectClipRectangle( hDC, &rcRect) != RGN_ERROR);
    FREEPS;                       //Kill BUSY thread.
    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");
    SUBEXIT();

    SUBENTER(PMERR_INV_RECT, EntryName);
    rcRect.xRight = -pg_width;
    bSuccess = (GreIntersectClipRectangle( hDC, &rcRect) != RGN_ERROR);
    CHECKRC(bSuccess,PMERR_INV_RECT,"");
    SUBEXIT();

//    SUBENTER(PMERR_COORDINATE_OVERFLOW, EntryName);
//    rcRect.xRight = 134217728L;
//    bSuccess = (GreIntersectClipRectangle( hDC, &rcRect) != RGN_ERROR);
//    CHECKRC(bSuccess,PMERR_COORDINATE_OVERFLOW,"");
//    SUBEXIT();

    SUBENTER(PMERR_INV_COORDINATE, EntryName);
    // determine maximum valid world coordinate plus one device unit
    rcRect.xRight = 32768L;
    rcRect.yTop = 32768L;
    if(GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L, (PPOINTL)&rcRect))
    {
      bSuccess = (GreIntersectClipRectangle( hDC, &rcRect) != RGN_ERROR);
      CHECKRC(bSuccess, PMERR_INV_COORDINATE, "");
    }
    else
    {
      // clear the DTT error in the queue
      WinGetLastError(hAB);
      LOGINFO(L_TRACE, "*",
               "PMERR_INV_COORDINATE skipped! Cannot create w/configuration");
    }
    SUBEXIT();

    // cleanup
    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
      SHOWERROR("GpiSetClipRegion", NULL);
    if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreExcludeClipRectangleRc
 *
 * Purpose: return code test for GreExcludeClipRectangle function
 *
 * System Requirements: Section 13.5.9
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GreExcludeClipRectangle, GpiSetClipRegion,
 *    GpiDestroyRegion, GpiBeginArea, GpiEndArea, GpiBeginPath, GpiEndPath,
 *    MAKEPSBUSY, FREEPS, and DeletePrinterOutput.
 *
 * Limitations:
 *          could not generate PMERR_COORDINATE_OVERFLOW, test omitted.
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - initialize a rectangle to the page size
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - start an area (GpiBeginArea)
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_IN_AREA
 *      - end the area (GpiEndArea)
 *      - start a path (GpiBeginPath)
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_IN_PATH
 *      - end the path (GpiEndPath)
 *      - Call MAKEPSBUSY then the function under test, then FREEPS and
 *        pass function call result to CHECKRC to log return code check
 *        for PMERR_HDC_BUSY.
 *      - set rectangle to have xRight < xLeft
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_RECT
 *      - set rectangle upper right corner to be world space equivalent to
 *        one more than max valid device coordinate (use GpiConvert from
 *        device to world of 32768)
 *      - if GpiConvert succeeded
 *        - call the function under test with this point and pass result to
 *          CHECKRC to log return code check for PMERR_INV_COORDINATE
 *        else
 *        - log that PMERR_INV_COORDINATE skipped as could not create the
 *          error for this device size and resolution.
 *      - cleanup: deselect any active region and destroy it.
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreExcludeClipRectangleRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreExcludeClipRectangleRc"; // entry point name
  HDC      hDC;
  HAB      hAB;
  BOOL     bSuccess;
  HRGN     hrgn;
  RECTL    rcRect;
//  POINTL   ptlWorldSize;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
          if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    // initialize to something valid for INV_HDC and HDC_BUSY tests
    rcRect.xLeft = 0L;
    rcRect.xRight = pg_width - 1;
    rcRect.yBottom = 0L;
    rcRect.yTop = pg_height - 1;

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_INV_IN_AREA
        PMERR_INV_IN_PATH
        PMERR_HDC_BUSY
        PMERR_DEV_FUNC_NOT_INSTALLED - not tested
        PMERR_COORDINATE_OVERFLOW
        PMERR_INSUFFICIENT_MEMORY
        PMERR_INV_LENGTH_OR_COUNT
        PMERR_INV_RECT
        PMERR_INV_COORDINATE
        PMERR_INV_HDC
        PMERR_INV_HRGN
        PMERR_INV_REGION_CONTROL
        PMERR_INV_COORD_SPACE
      *******************************************************************/
    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = (GreExcludeClipRectangle( (HDC)NULL, &rcRect) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = (GreExcludeClipRectangle( (HDC)-1L, &rcRect) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_IN_AREA, EntryName);
    if (GpiBeginArea(hPS, BA_WINDING) == FALSE)
    {
      /* setup errored */
      ErrorCount++;
      ERRSTATUS(TRUE,"GpiBeginArea", NULL);
    }
    else
    {
      bSuccess = (GreExcludeClipRectangle(hDC, &rcRect) != RGN_ERROR);
      CHECKRC(bSuccess, PMERR_INV_IN_AREA, NULL);
      if (GpiEndArea(hPS) == FALSE)
      {
        /* close up failed */
        ErrorCount++;
        ERRSTATUS(TRUE,"GpiEndArea", NULL);
      }
    }
    SUBEXIT();

    SUBENTER(PMERR_INV_IN_PATH, EntryName);
    if (GpiBeginPath(hPS, 1L) == FALSE)
    {
      /* setup errored */
      ErrorCount++;
      ERRSTATUS(TRUE,"GpiBeginPath", NULL);
    }
    else
    {
      bSuccess = (GreExcludeClipRectangle(hDC, &rcRect) != RGN_ERROR);
      CHECKRC(bSuccess, PMERR_INV_IN_PATH, NULL);
      if (GpiEndPath(hPS) == FALSE)
      {
        /* close up failed */
        ErrorCount++;
        ERRSTATUS(TRUE,"GpiEndPath", NULL);
      }
    }
    SUBEXIT();

    SUBENTER(PMERR_HDC_BUSY, EntryName);
    MAKEPSBUSY(hPS);               //Make this PS busy.
    bSuccess = (GreExcludeClipRectangle( hDC, &rcRect) != RGN_ERROR);
    FREEPS;                       //Kill BUSY thread.
    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");
    SUBEXIT();

    SUBENTER(PMERR_INV_RECT, EntryName);
    rcRect.xRight = -pg_width;
    bSuccess = (GreExcludeClipRectangle( hDC, &rcRect) != RGN_ERROR);
    CHECKRC(bSuccess,PMERR_INV_RECT,"");
    SUBEXIT();

//    SUBENTER(PMERR_COORDINATE_OVERFLOW, EntryName);
//    rcRect.xRight = 134217728L;
//    bSuccess = (GreExcludeClipRectangle( hDC, &rcRect) != RGN_ERROR);
//    CHECKRC(bSuccess,PMERR_COORDINATE_OVERFLOW,"");
//    SUBEXIT();

    SUBENTER(PMERR_INV_COORDINATE, EntryName);
    // determine maximum valid world coordinate plus one device unit
    rcRect.xRight = 32768L;
    rcRect.yTop = 32768L;
    if(GpiConvert(hPS, CVTC_DEVICE, CVTC_WORLD, 2L, (PPOINTL)&rcRect))
    {
      bSuccess = (GreExcludeClipRectangle( hDC, &rcRect) != RGN_ERROR);
      CHECKRC(bSuccess, PMERR_INV_COORDINATE, "");
    }
    else
    {
      // clear the DTT error in the queue
      WinGetLastError(hAB);
      LOGINFO(L_TRACE, "*",
               "PMERR_INV_COORDINATE skipped! Cannot create w/configuration");
    }
    SUBEXIT();

    // cleanup
    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
      SHOWERROR("GpiSetClipRegion", NULL);
    if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreSetXformRectRc
 *
 * Purpose: return code test for GreSetXformRectRc
 *
 * System Requirements: Section 13.5.10
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GreSetXformRect, MAKEPSBUSY, FREEPS, and
 *    DeletePrinterOutput.
 *
 * Limitations:
 *          NONE but could not generate PMERR_INV_COORDINATE - all values
 *          from -134217729 to +134217728 always obtained "worked properly
 *          instead" from CHECKRC. Also tried other techniques like changing
 *          page size, viewport size, and setting transforms. Nothing worked
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - initialize a rectangle to the device size
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - start an area (GpiBeginArea)
 *      - Call MAKEPSBUSY then the function under test, then FREEPS and
 *        pass function call result to CHECKRC to log return code check
 *        for PMERR_HDC_BUSY.
 *      - set rectangle to have xRight < xLeft
 *      - call the function under test and pass result to CHECKRC to log
 *        return code check for PMERR_INV_RECT
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreSetXformRectRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreSetXformRectRc"; // entry point name
  HDC   hDC;
  HAB   hAB;
  BOOL  bSuccess;
  RECTL rcRect;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
          if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    // initialize to something valid for INV_HDC and HDC_BUSY tests
    rcRect.xLeft = 0L;
    rcRect.xRight = dev_width - 1;
    rcRect.yBottom = 0L;
    rcRect.yTop = dev_height - 1;

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_HDC_BUSY
        PMERR_DEV_FUNC_NOT_INSTALLED - not tested
        PMERR_INSUFFICIENT_MEMORY
        PMERR_INV_RECT
        PMERR_INV_COORDINATE
        PMERR_INV_HDC
        PMERR_INV_HRGN
        PMERR_INV_REGION_CONTROL
      *******************************************************************/
    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = (GreSetXformRect( (HDC)NULL, &rcRect) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = (GreSetXformRect( (HDC)-1L, &rcRect) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_HDC_BUSY, EntryName);
    MAKEPSBUSY(hPS);               //Make this PS busy.
    bSuccess = (GreSetXformRect( hDC, &rcRect) != RGN_ERROR);
    FREEPS;                       //Kill BUSY thread.
    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");
    SUBEXIT();

    SUBENTER(PMERR_INV_RECT, EntryName);
    rcRect.xRight = -dev_width;
    bSuccess = (GreSetXformRect( hDC, &rcRect) != RGN_ERROR);
    CHECKRC(bSuccess,PMERR_INV_RECT,"");
    SUBEXIT();

//    SUBENTER(PMERR_INV_COORDINATE, EntryName);
//    rcRect.xRight = 134217728L;
//    bSuccess = (GreSetXformRect( hDC, &rcRect) != RGN_ERROR);
//    CHECKRC(bSuccess,PMERR_INV_COORDINATE,"");
//    SUBEXIT();

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreSaveRegionRc
 *
 * Purpose: return code test for GreSaveRegion function
 *
 * System Requirements: Section 13.5.11
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GreSaveRegion, MAKEPSBUSY, FREEPS, and
 *    DeletePrinterOutput.
 *
 * Limitations:
 *          NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - set save level variable to 1L
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - start an area (GpiBeginArea)
 *      - Call MAKEPSBUSY then the function under test, then FREEPS and
 *        pass function call result to CHECKRC to log return code check
 *        for PMERR_HDC_BUSY.
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreSaveRegionRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreSaveRegionRc"; // entry point name
  HDC   hDC;
  HAB   hAB;
  BOOL  bSuccess;
  LONG  cSave;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
          if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    cSave = 1L;

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_HDC_BUSY
        PMERR_INSUFFICIENT_MEMORY
        PMERR_INV_RECT
        PMERR_INV_COORDINATE
        PMERR_INV_HDC
        PMERR_INV_HRGN
      *******************************************************************/
    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = GreSaveRegion( (HDC)NULL, cSave);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = GreSaveRegion( (HDC)-1L, cSave);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_HDC_BUSY, EntryName);
    MAKEPSBUSY(hPS);               //Make this PS busy.
    bSuccess = GreSaveRegion( hDC, cSave);
    FREEPS;                       //Kill BUSY thread.
    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");
    SUBEXIT();

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreRestoreRegionRc
 *
 * Purpose: return code test for GreRestoreRegion function
 *
 * System Requirements: Section 13.5.12
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GreRestoreRegion, MAKEPSBUSY, FREEPS, and
 *    DeletePrinterOutput.
 *
 * Limitations:
 *          NONE
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - set restore level variable to 0L
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - start an area (GpiBeginArea)
 *      - Call MAKEPSBUSY then the function under test, then FREEPS and
 *        pass function call result to CHECKRC to log return code check
 *        for PMERR_HDC_BUSY.
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreRestoreRegionRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreRestoreRegionRc"; // entry point name
  HDC   hDC;
  HAB   hAB;
  BOOL  bSuccess;
  LONG  cSave;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
          if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    cSave = 0L;

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_HDC_BUSY
        PMERR_INSUFFICIENT_MEMORY
        PMERR_INV_RECT
        PMERR_INV_COORDINATE
        PMERR_INV_HDC
        PMERR_INV_HRGN
      *******************************************************************/
    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = GreRestoreRegion( (HDC)NULL, cSave);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = GreRestoreRegion( (HDC)-1L, cSave);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

//Removed to prevent TRAP
//    SUBENTER(PMERR_HDC_BUSY, EntryName);
//    MAKEPSBUSY(hPS);               //Make this PS busy.
//    bSuccess = GreRestoreRegion( hDC, cSave);
//    FREEPS;                       //Kill BUSY thread.
//    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");
//    SUBEXIT();

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreSelectPathRegionRc
 *
 * Purpose: return code test for GreSelectPathRegion function
 *
 * System Requirements: Section 13.5.13
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GreSelectPathRegion, GpiBeginPath, GpiEndPath,
 *    GpiSetClipPath, GpiSetClipRegion, GpiDestroyRegion, GpiQueryClipRegion,
 *    GpiIntersectClipRectangle, MAKEPSBUSY, FREEPS, and DeletePrinterOutput.
 *
 * Limitations:
 *          PMERR_HDC_BUSY test causes DTT to TRAP D. test omitted
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - initialize points and rectangle to some minimal values
 *      - create a path using the points, select the path (GpiSetClipPath)
 *        and save in a region handle (GpiSetClipRegion)
 *      - create a clip region using rectangle that matches point coords.
 *        (GpiIntersectClipRectangle)
 *      - obtain a region handle for the clip region (GpiQueryClipRegion)
 *      - call the function under test with this region and pass the result
 *        to CHECKRC to log return code check for PMERR_REGION_IS_CLIP_REGION
 *      - deselect and destroy the clip region
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HRGN to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HRGN
 *      - cleanup: discard the clip path (GpiSetClipPath), destroy the path
 *        (GreSelectClipRegion), deselect and destroy the clip region
 *        (GpiSetClipRegion and GpiDestroyRegion).
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreSelectPathRegionRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreSelectPathRegionRc"; // entry point name
  HDC    hDC;
  HAB    hAB;
  BOOL   bSuccess;
  HRGN   hrgnP, hrgnC;
  POINTL ptlPathLLC, ptlPathURC;
  RECTL  rcRect;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
          if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    rcRect.xLeft = rcRect.yBottom = ptlPathLLC.x = ptlPathLLC.y = 100L;
    rcRect.xRight = rcRect.yTop = ptlPathURC.x = ptlPathURC.y = 200L;

    // create the path
    if (GpiBeginPath(hPS, 1L) == FALSE)
    {
      SHOWERROR("GpiBeginPath", EntryName);
    }

    GpiMove(hPS, &ptlPathLLC);
    GpiBox(hPS, DRO_OUTLINE, &ptlPathURC, 0L, 0L);

    if (GpiEndPath(hPS) == FALSE)
    {
      SHOWERROR("GpiEndPath", EntryName);
    }

    // establish the path as the clip path
    if (GpiSetClipPath(hPS, 1L, SCP_ALTERNATE | SCP_AND) == FALSE)
    {
      SHOWERROR("GpiSetClipPath", EntryName);
    }

    // save region handle and set to no region selected
    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgnP) == RGN_ERROR)
      SHOWERROR("GpiSetClipRegion", NULL);

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_HDC_BUSY
        PMERR_DEV_FUNC_NOT_INSTALLED - not tested
        PMERR_INSUFFICIENT_MEMORY
        PMERR_INV_RECT
        PMERR_INV_COORDINATE
        PMERR_INV_HDC
        PMERR_INV_HRGN
        PMERR_HRGN_BUSY
        PMERR_REGION_IS_CLIP_REGION
        PMERR_INV_REGION_CONTROL
      *******************************************************************/

    SUBENTER(PMERR_REGION_IS_CLIP_REGION, EntryName);
    if (GpiIntersectClipRectangle(hPS, &rcRect) == RGN_ERROR)
      SHOWERROR("GpiIntersectClipRectangle", NULL);
    hrgnC = GpiQueryClipRegion(hPS);
    if (hrgnC == RGN_ERROR)
      SHOWERROR("GpiQueryClipRegion", NULL);
    bSuccess = GreSelectPathRegion( hDC, hrgnC);
    CHECKRC(bSuccess,PMERR_REGION_IS_CLIP_REGION,"");

    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgnC) == RGN_ERROR)
      SHOWERROR("GpiSetClipRegion", NULL);
    if (hrgnC != NULLHANDLE && GpiDestroyRegion(hPS, hrgnC) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);
    SUBEXIT();

    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = GreSelectPathRegion( (HDC)NULL, hrgnP);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = GreSelectPathRegion( (HDC)-1L, hrgnP);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HRGN, EntryName);
    bSuccess = GreSelectPathRegion( hDC, (HRGN) -1L);
    CHECKRC(bSuccess,PMERR_INV_HRGN,"");
    SUBEXIT();

// Removed as this function causes DTT to TRAP D.
//    SUBENTER(PMERR_HDC_BUSY, "Negative HDC Test");
//    bSuccess = GreSelectPathRegion( hDC, hrgnP);
//    CHECKRC(bSuccess, PMERR_HDC_BUSY, "");
//    SUBEXIT();

    // discard the clip path - so can create another one!
    if (GpiSetClipPath(hPS, 0L, SCP_RESET) == FALSE)
    {
      SHOWERROR("GpiSetClipPath", EntryName);
    }

    // reset to no clip path
    if (GreSelectPathRegion(hDC, NULL) == FALSE)
    {
      ErrorCount++;
      ERRSTATUS(TRUE, "GreSelectPathRegion:", "reset");
    }

    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgnP) == RGN_ERROR)
      SHOWERROR("GpiSetClipRegion", NULL);
    if (hrgnP != NULLHANDLE && GpiDestroyRegion(hPS, hrgnP) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreRegionSelectBitmapRc
 *
 * Purpose: return code test for GreRegionSelectBitmap function
 *
 * System Requirements: Section 13.5.14
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GreRegionSelectBitmap, and DeletePrinterOutput.
 *
 * Limitations:
 *          PMERR_HDC_BUSY test omitted due to MAKEPSBUSY implementation
 *          which does a GpiBitBlt from a memory device associated HPS to
 *          the HPS used for testing. This will fail as this function
 *          requires an HPS associated to a memory DC and a BitBlt from
 *          memory DC to memory DC crashes.
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreRegionSelectBitmapRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreRegionSelectBitmapRc"; // entry point name
  HDC   hDC;
  HAB   hAB;
  BOOL  bSuccess;
  HBITMAP hbm;                  //Bitmap handle

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
          if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

// Perform the setting up of the test environment.

    SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

    SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_HDC_BUSY
        PMERR_DEV_FUNC_NOT_INSTALLED - not tested
        PMERR_INSUFFICIENT_MEMORY
        PMERR_INV_RECT
        PMERR_INV_COORDINATE
        PMERR_INV_HDC
        PMERR_INV_HRGN
        PMERR_INV_REGION_CONTROL
      *******************************************************************/

    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess = (BOOL)GreRegionSelectBitmap( (HDC)NULL, hbm);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess = (BOOL)GreRegionSelectBitmap( (HDC)-1L, hbm);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

// Removed as busy function would do a GpiBitBlt from a memory to a memory DC
// which is invalid. Cannot test this error code without rewriting busy code.
// Would also have to create a new "MEMORY" DC and associated PS.
//    SUBENTER(PMERR_HDC_BUSY, "Negative HDC Test");
//    MAKEPSBUSY(hpsMemory);
//    // select the bitmap region (function undertest);
//    bSuccess = GreRegionSelectBitmap(hdcMemory, hBM2);
//    FREEPS();
//    CHECKRC(bSuccess, PMERR_HDC_BUSY, "");
//    SUBEXIT();

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreCopyClipRegionRc
 *
 * Purpose: Return code test for GreCopyClipRegion function
 *
 * System Requirements: Section 13.5.15
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN, EXIT_MAIN, SETENTER, SETEXIT, SUBENTER, SUBEXIT, CHECKRC,
 *    WinSendMsg, WinQueryAnchorBlock, WinGetLastError, SHOWERROR, CLEANUP,
 *    GpiQueryDevice, GreCopyClipRegion, GpiIntersectClipRectangle,
 *    GpiSetClipRegion, GpiDestroyRegion, and DeletePrinterOutput.
 *
 * Limitations:
 *          PMERR_HDC_BUSY test fails but left in (it may work sometimes
 *          to some devices)
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      - Query window handle for the anchor block
 *      - Use the anchor block handle to reset the error queue incase there
 *        are any left from previous testing or by DTT.
 *      - Pass a NULL HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HDC to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HDC
 *      - Pass a -1L HRGN to the function under test and pass result to
 *        CHECKRC to log return code check for PMERR_INV_HRGN
 *      - create two page sized clip regions (GpiIntersectClipRectangle) and
 *        save one in a region handle (GpiSetClipRegion) and leave the other
 *        active.
 *      - call MAKEPSBUSY then try to copy the second region over the active
 *        call FREEPS and pass result of copy attempt to CHECKRC to log
 *        return code check for PMERR_HDC_BUSY.
 *      - destroy the two regions.
 *      - create a clip region using a page sized rectangle
 *        (GpiIntersectClipRectangle)
 *      - obtain a region handle for the clip region (GpiQueryClipRegion)
 *      - call the function under test with this region and pass the result
 *        to CHECKRC to log return code check for PMERR_REGION_IS_CLIP_REGION
 *      - deselect and destroy the clip region
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreCopyClipRegionRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreCopyClipRegionRc"; // entry point name
  HDC   hDC;
  HAB   hAB;
  BOOL  bSuccess;
  HRGN  hrgn;
  RECTL rcRect;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
          if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

     SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

     SETEXIT();

    /*********************************************************************
      Return Codes:
        PMERR_HDC_BUSY
        PMERR_DEV_FUNC_NOT_INSTALLED - not tested
        PMERR_INSUFFICIENT_MEMORY
        PMERR_INV_RECT
        PMERR_INV_COORDINATE
        PMERR_INV_HDC
        PMERR_INV_HRGN
        PMERR_HRGN_BUSY
        PMERR_REGION_IS_CLIP_REGION
      *******************************************************************/

    // test for PMERR_INV_HDC
    SUBENTER(PMERR_INV_HDC, "NULL HDC Test");
    // Pass an invalid HDC , hDC=NULL
    bSuccess =
     (GreCopyClipRegion((HDC)NULL, hrgn, NULL, COPYCRGN_CLIPRGN) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "NULL HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HDC, "Negative HDC Test");
    // Pass an invalid HDC , HDC=-1L
    bSuccess =
     (GreCopyClipRegion((HDC)-1L, hrgn, NULL, COPYCRGN_CLIPRGN) != RGN_ERROR);
    CHECKRC(bSuccess, PMERR_INV_HDC, "Negative HDC Test");
    SUBEXIT();

    SUBENTER(PMERR_INV_HRGN, EntryName);
    hrgn = (HRGN)-1L;
    bSuccess =
         (GreCopyClipRegion( hDC, hrgn, NULL, COPYCRGN_CLIPRGN) != RGN_ERROR);
    CHECKRC(bSuccess,PMERR_INV_HRGN,"");
    SUBEXIT();

    SUBENTER(PMERR_HDC_BUSY, EntryName);

    // create a clip region
    rcRect.xLeft = 0L;
    rcRect.xRight = pg_width - 1;
    rcRect.yBottom = 0L;
    rcRect.yTop = pg_height - 1;
    if (GpiIntersectClipRectangle(hPS, &rcRect) == RGN_ERROR)
      SHOWERROR("GpiIntersectClipRectangle", NULL);
          if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
      SHOWERROR("GpiSetClipRegion", NULL);

    // create a clip region
    rcRect.xLeft = 0L;
    rcRect.xRight = pg_width - 1;
    rcRect.yBottom = 0L;
    rcRect.yTop = pg_height - 1;
    if (GpiIntersectClipRectangle(hPS, &rcRect) == RGN_ERROR)
      SHOWERROR("GpiIntersectClipRectangle", NULL);

    MAKEPSBUSY(hPS);               //Make this PS busy.
    bSuccess =
      (GreCopyClipRegion( hDC, hrgn, &rcRect, COPYCRGN_CLIPRGN) != RGN_ERROR);
    FREEPS;                       //Kill BUSY thread.
    CHECKRC(bSuccess,PMERR_HDC_BUSY,"");

    // cleanup
          if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);
    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
      SHOWERROR("GpiSetClipRegion", NULL);
    if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);

    SUBEXIT();

    SUBENTER(PMERR_REGION_IS_CLIP_REGION, EntryName);

    // create a clip region
    rcRect.xLeft = MapX(100L);
    rcRect.xRight = MapX(300L);
    rcRect.yBottom = MapY(200L);
    rcRect.yTop = MapY(400L);
    if (GpiIntersectClipRectangle(hPS, &rcRect) == RGN_ERROR)
      SHOWERROR("GpiIntersectClipRectangle", NULL);
    hrgn = GpiQueryClipRegion(hPS);
    if (hrgn == RGN_ERROR)
      SHOWERROR("GpiQueryClipRegion", NULL);

    bSuccess =
         (GreCopyClipRegion( hDC, hrgn, NULL, COPYCRGN_CLIPRGN) != RGN_ERROR);
    CHECKRC(bSuccess,PMERR_REGION_IS_CLIP_REGION,"");

    // cleanup
    if (GpiSetClipRegion(hPS, NULLHANDLE, &hrgn) == RGN_ERROR)
      SHOWERROR("GpiSetClipRegion", NULL);
    if (hrgn != NULLHANDLE && GpiDestroyRegion(hPS, hrgn) == RGN_ERROR)
      SHOWERROR("GpiDestroyRegion", NULL);

    SUBEXIT();

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

/****************************************************************************
 *
 * Routine Name: GreSelectPathCurvesRc
 *
 * Purpose: Return code test for GreSelectPathCurves function
 *
 * System Requirements: Section 13.5.16
 *
 * Revision Log: 01/29/92,    Graphics Suite Extension, Thom Bain
 *                      Initial Entry Point Design.
 *
 * Inputs:
 *
 *    hPS            // presentation space
 *    hWndClient     // client window
 *    BOOL           // selection call
 *
 * Outputs:
 *          NONE
 *
 * Subroutines Required:
 *    ENTER_MAIN()
 *    EXIT_MAIN()
 *    SETENTER()
 *    SETEXIT()
 *    SUBENTER()
 *    SUBEXIT()
 *    CHECKRC()
 *    All other required subroutines are listed here.
 *
 * Limitations:
 *          FUNCTION STUBBED DUE TO LACK OF DOCUMENTATION
 *
 *
 ***************************************************************************/

/***************************************************************************
 * Algorithm:
 * {
 *      Detailed Design in algorithmic form
 * }
 *
 ***************************************************************************/

VOID  APIENTRY GreClipPathCurvesRc(HPS hPS, HWND hWndClient, BOOL SelectionCall)
{
  static PCHAR EntryName="GreClipPathCurvesRc"; // entry point name
  HDC   hDC;
  HAB   hAB;

  if (SelectionCall)
    WinSendMsg (hWndClient, WM_DTHREAD_DOES_DEVCALLS, NULL, NULL);
  else
  {
    //Get Anchor Block.
          if ((hAB=WinQueryAnchorBlock(hWndClient))==NULLHANDLE){
        SHOWERROR("WinQueryAnchorBlock",EntryName);
        return;     //Can't continue without this AB.
    }

    // clear any left over DTT errors in the queue
    WinGetLastError(hAB);

    ENTER_MAIN(EntryName);

     SETENTER(EntryName,"Setup");

    // Get the Device Context Handle from the hPS.
    if ( (hDC=GpiQueryDevice(hPS)) == HDC_ERROR)
    {
      CLEANUP("GpiQueryDevice", EntryName);
      return;
    }

     SETEXIT();

    // Delete the printer output, so a blank page is not printed.
    DeletePrinterOutput(EntryName);

    // Exit the entry point
    EXIT_MAIN(EntryName);
  }
  return;
}

